/*------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG, 2021

   eqdbobj.h - Eloquence database binary object utility functions
   $Id: eqdbobj.h,v 29.3 2023/12/08 17:08:30 rhg Exp $

   Edit History:
   15.06.2021, initial release

------------------------------------------------------------------------*/

#if !defined(DBOBJ_INCLUDED)
#define DBOBJ_INCLUDED

#define DBOBJ_VER 0x8400

#if defined(__cplusplus)
extern "C" {
#endif

/*
   Win32 API calling convention
   default: __stdcall, use -DDBCALL3K=__cdecl compiler directive otherwise
*/
#if defined(_WIN32)
#if !defined(DBOBJ_CALL)
#define DBOBJ_CALL __stdcall
#endif
#else
#if defined(DBOBJ_CALL)
#undef DBOBJ_CALL
#endif
#define DBOBJ_CALL /**/
#endif

int DBOBJ_CALL idb_obj_ctrl(int mode, int nval, int *oval);

int DBOBJ_CALL idb_obj_putdata(int mode, const void *data, size_t data_sz);
int DBOBJ_CALL idb_obj_getdata(int mode, void *buf, size_t buf_sz, size_t *len);

int DBOBJ_CALL idb_obj_fetch(
   int dbid, const void *set, unsigned int id, size_t *len_p);

int DBOBJ_CALL idb_obj_delete(
   int dbid, const void *set, unsigned int id);

int DBOBJ_CALL idb_obj_store(
   int dbid, const void *set, int mode, unsigned int id);

const char *DBOBJ_CALL idb_obj_msg(int stat);

enum DBOBJ_STATUS {
   DBOBJ_OK = 0,
   DBOBJ_ERR_DB = -1,          /* unexpected db status */
   DBOBJ_ERR_SET = -2,         /* invalid or improper data set */
   DBOBJ_NOTFOUND = 3,         /* object not found */
   DBOBJ_EXISTS = 4,           /* object already exists */
   DBOBJ_ERR_LOCK = -5,        /* database locking failed */
   DBOBJ_ERR_MEMALLOC = -6,    /* memory allocation failed */
   DBOBJ_ERR_CORRUPT = -7,     /* object data inconsistent */
   DBOBJ_ERR_ZLIB = -8,        /* zlib call failed */
   DBOBJ_ERR_BADMODE = -9,     /* bad mode value */
   DBOBJ_ERR_BADARG  = -10,    /* bad argument value */
   DBOBJ_ERR_NODATA = 11,      /* no data passed (sequence error) */
   DBOBJ_ERR_LIMIT = 12        /* data size exceeds limit */
};

void DBOBJ_CALL dbobjctrl(
  const void *mode, void *status, const void *nval, void *oval);

void DBOBJ_CALL dbobjputdata(
  const void *mode, void *status, const void *data, const void *len);
void DBOBJ_CALL dbobjgetdata(
  const void *mode, void *status, void *buf, const void *buf_sz, 
  void *len);

void DBOBJ_CALL dbobjfetch(
   const void *base, const void *dset, void *status, 
   const void *id, void *len);

void DBOBJ_CALL dbobjdelete(
   const void *base, const void *dset, void *status,
   const void *id);

void DBOBJ_CALL dbobjstore(
   const void *base, const void *dset, const void *mode,
   void *status, const void *id);

#if defined(_WIN32) && !DBOBJ_LIB
extern __declspec(dllimport) const char dbobj__version_tag[];
extern __declspec(dllimport) const char dbobj__copyright[];
extern __declspec(dllimport) const char dbobj__patchlevel[];
#else
extern const char dbobj__version_tag[];
extern const char dbobj__copyright[];
extern const char dbobj__patchlevel[];
#endif

#if defined(__cplusplus)
}
#endif

#endif /* DBOBJ_INCLUDED */
